#include <iostream>

template< typename T, T v >
struct integral_constant {
	static constexpr T value = v;
};

using true_type = integral_constant< bool, true >;
using false_type = integral_constant< bool, false >;

template< typename T >
struct is_pointer : false_type {};

template< typename T >
struct is_pointer<T*> : true_type {};

template< typename T >
inline constexpr bool is_pointer_v = is_pointer<T>::value;

template< typename T >
struct remove_pointer {
	using type = T;
};

template< typename T >
struct remove_pointer<T*> {
	using type = T;
};

template< typename T >
using remove_pointer_t = typename remove_pointer<T>::type;

int main() {
	static_assert( is_pointer<int*>::value );
	static_assert( is_pointer_v<int*> );

	std::cout << std::boolalpha << is_pointer_v<bool> << '\n';

	remove_pointer_t<int*> value = 3;
}
